(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: otype.mli,v 1.18 2008/02/11 23:11:03 weis Exp $ *)

(** {3 Functions on OCaml types.} *)

open Parsetree
;;

(** Functions on generators. *)

val name_of_longident : Longident.t -> string
;;
val name_of_generator : generator -> string
;;
val qualified_name_of_longident : Longident.t -> string
;;
val qualified_name_of_generator : generator -> string
;;

(** Functions on core types. *)

val constant_type : string -> core_type
;;

val type_of_eq : string -> string list -> core_type
(** The types of the equality function for the type name argument. *)
;;

val projection_type : string -> string list -> core_type -> core_type
;;
val injection_type : string -> string list -> core_type -> core_type
(** The types of the injection and projection functions
    for type abbreviations. *)
;;

val is_list_type : core_type -> bool
;;

val list_argument : core_type -> core_type
(** Returns the argument of a list type. Raises Invalid_argument if the argument is not a list type. *)

(* Functions on type declarations. *)

val comment_relations_in_type_declaration :
  Parsetree.type_declaration -> type_declaration
(** To encapsulate relations in a [Comment] annotation. *)
;;

val make_public_type_declaration : type_declaration -> type_declaration
;;

val get_structure_items_of_type_declaration :
  type_declaration -> Parsetree.structure_item list
;;
