(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Laura Lowenthal, projet Protheo, INRIA Lorraine           *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: parsetree_utils.ml,v 1.1 2008/02/13 10:58:59 lowenthl Exp $ *)

open Parsetree
open Funutils
;;

(** Gets the core type description of a core type *)
let type_desc c_type = c_type.ptyp_desc
;;

let change_type_desc core_type desc =
  {ptyp_loc = core_type.ptyp_loc; ptyp_desc = desc}
;;

let subst param_binding types =
  let rec app_subst core_type = 
    let new_desc = change_type_desc core_type in
    match type_desc core_type with
      Ptyp_any -> core_type
    | Ptyp_var a ->
	(try List.assoc a param_binding with Not_found -> core_type)
    | Ptyp_arrow (l, t1, t2) ->
	new_desc ^$ Ptyp_arrow (l, app_subst t1, app_subst t2)
    | Ptyp_tuple ts -> new_desc ^$ Ptyp_tuple (List.map app_subst ts)
    | Ptyp_constr (name, ts) ->
	new_desc ^$ Ptyp_constr (name, List.map app_subst ts)
    | Ptyp_object _ -> core_type
    | Ptyp_class _ -> core_type
    | Ptyp_alias (t, s) -> new_desc ^$ Ptyp_alias (app_subst t, s)
    | Ptyp_variant _ -> core_type
    | Ptyp_poly _ -> core_type
    | Ptyp_parens t -> new_desc ^$ Ptyp_parens (app_subst t) in
  List.map app_subst types
